@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Codership Oy <info@codership.com>
+ * Copyright (C) 2012-2017 Codership Oy <info@codership.com>
  */
 
 #include "asio_tcp.hpp"
@@ -14,9 +14,7 @@ gcomm::AsioTcpSocket::AsioTcpSocket(AsioProtonet& net, const gu::URI& uri)
     Socket       (uri),
     net_         (net),
     socket_      (net.io_service_),
-#ifdef HAVE_ASIO_SSL_HPP
     ssl_socket_  (0),
-#endif /* HAVE_ASIO_SSL_HPP */
     send_q_      (),
     recv_buf_    (net_.mtu() + NetHeader::serial_size_),
     recv_offset_ (0),
@@ -31,10 +29,8 @@ gcomm::AsioTcpSocket::~AsioTcpSocket()
 {
     log_debug << "dtor for " << id();
     close_socket();
-#ifdef HAVE_ASIO_SSL_HPP
     delete ssl_socket_;
     ssl_socket_ = 0;
-#endif /* HAVE_ASIO_SSL_HPP */
 }
 
 void gcomm::AsioTcpSocket::failed_handler(const asio::error_code& ec,
@@ -65,14 +61,19 @@ void gcomm::AsioTcpSocket::failed_handler(const asio::error_code& ec,
     }
 }
 
-#ifdef HAVE_ASIO_SSL_HPP
 void gcomm::AsioTcpSocket::handshake_handler(const asio::error_code& ec)
 {
     if (ec)
     {
-        log_error << "handshake with remote endpoint "
-                  << remote_addr() << " failed: " << ec << ": '" << ec.message()
-                  << "' ( " << gu::extra_error_info(ec) << ")";
+        if (ec.category() == asio::error::get_ssl_category() &&
+            gu::exclude_ssl_error(ec) == false)
+        {
+
+            log_error << "handshake with remote endpoint "
+                      << remote_addr() << " failed: " << ec << ": '"
+                      << ec.message()
+                      << "' ( " << gu::extra_error_info(ec) << ")";
+        }
         FAILED_HANDLER(ec);
         return;
     }
@@ -87,16 +88,18 @@ void gcomm::AsioTcpSocket::handshake_handler(const asio::error_code& ec)
         return;
     }
 
+    const char* compression_name = gu::compression(*ssl_socket_);
+
     log_info << "SSL handshake successful, "
              << "remote endpoint " << remote_addr()
              << " local endpoint " << local_addr()
              << " cipher: " << gu::cipher(*ssl_socket_)
-             << " compression: " << gu::compression(*ssl_socket_);
+             << " compression: "
+             << (compression_name != NULL ? compression_name : "none");
     state_ = S_CONNECTED;
     net_.dispatch(id(), Datagram(), ProtoUpMeta(ec.value()));
     async_receive();
 }
-#endif /* HAVE_ASIO_SSL_HPP */
 
 void gcomm::AsioTcpSocket::connect_handler(const asio::error_code& ec)
 {
@@ -114,7 +117,6 @@ void gcomm::AsioTcpSocket::connect_handler(const asio::error_code& ec)
             assign_local_addr();
             assign_remote_addr();
             set_socket_options();
-#ifdef HAVE_ASIO_SSL_HPP
             if (ssl_socket_ != 0)
             {
                 log_debug << "socket " << id() << " connected, remote endpoint "
@@ -129,7 +131,6 @@ void gcomm::AsioTcpSocket::connect_handler(const asio::error_code& ec)
             }
             else
             {
-#endif /* HAVE_ASIO_SSL_HPP */
                 log_debug << "socket " << id() << " connected, remote endpoint "
                           << remote_addr() << " local endpoint "
                           << local_addr();
@@ -137,9 +138,7 @@ void gcomm::AsioTcpSocket::connect_handler(const asio::error_code& ec)
                 net_.dispatch(id(), Datagram(), ProtoUpMeta(ec.value()));
                 async_receive();
 
-#ifdef HAVE_ASIO_SSL_HPP
             }
-#endif /* HAVE_ASIO_SSL_HPP */
         }
     }
     catch (asio::system_error& e)
@@ -163,7 +162,6 @@ void gcomm::AsioTcpSocket::connect(const gu::URI& uri)
                   asio::ip::tcp::resolver::query::flags(0));
         asio::ip::tcp::resolver::iterator i(resolver.resolve(query));
 
-#ifdef HAVE_ASIO_SSL_HPP
         if (uri.get_scheme() == gu::scheme::ssl)
         {
             ssl_socket_ = new asio::ssl::stream<asio::ip::tcp::socket>(
@@ -178,22 +176,18 @@ void gcomm::AsioTcpSocket::connect(const gu::URI& uri)
         }
         else
         {
-#endif /* HAVE_ASIO_SSL_HPP */
-            const std::string bind_ip = uri.get_option(gcomm::Socket::OptIfAddr, "");
-            if (!bind_ip.empty()) {
+            const std::string bind_ip(uri.get_option(
+                                          gcomm::Socket::OptIfAddr, ""));
+            if (!bind_ip.empty())
+            {
                 socket_.open(i->endpoint().protocol());
-                asio::ip::tcp::endpoint ep(
-                    asio::ip::address::from_string(bind_ip),
-                    // connect from any port.
-                    0);
+                asio::ip::tcp::endpoint ep(gu::make_address(bind_ip), 0);
                 socket_.bind(ep);
             }
             socket_.async_connect(*i, boost::bind(&AsioTcpSocket::connect_handler,
                                                   shared_from_this(),
                                                   asio::placeholders::error));
-#ifdef HAVE_ASIO_SSL_HPP
         }
-#endif /* HAVE_ASIO_SSL_HPP */
         state_ = S_CONNECTING;
     }
     catch (asio::system_error& e)
@@ -225,10 +219,18 @@ void gcomm::AsioTcpSocket::close()
     }
 }
 
+// Enable to introduce random errors for write handler
+// #define GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
 
 void gcomm::AsioTcpSocket::write_handler(const asio::error_code& ec,
                                          size_t bytes_transferred)
 {
+#ifdef GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+    static const long empty_rate(10000);
+    static const long bytes_transferred_less_than_rate(10000);
+    static const long bytes_transferred_not_zero_rate(10000);
+#endif // GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+
     Critical<AsioProtonet> crit(net_);
 
     if (state() != S_CONNECTED && state() != S_CLOSING)
@@ -236,7 +238,8 @@ void gcomm::AsioTcpSocket::write_handler(const asio::error_code& ec,
         log_debug << "write handler for " << id()
                   << " state " << state();
 #ifdef HAVE_ASIO_SSL_HPP
-        if (ec.category() == asio::error::get_ssl_category())
+        if (ec.category() == asio::error::get_ssl_category() &&
+            gu::exclude_ssl_error(ec) == false)
         {
             log_warn << "write_handler(): " << ec.message()
                      << " (" << gu::extra_error_info(ec) << ")";
@@ -247,34 +250,70 @@ void gcomm::AsioTcpSocket::write_handler(const asio::error_code& ec,
 
     if (!ec)
     {
-        gcomm_assert(send_q_.empty() == false);
-        gcomm_assert(send_q_.front().len() >= bytes_transferred);
-
-        while (send_q_.empty() == false &&
-               bytes_transferred >= send_q_.front().len())
+        if (send_q_.empty() == true
+#ifdef GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+            || ::rand() % empty_rate == 0
+#endif // GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+            )
         {
-            const Datagram& dg(send_q_.front());
-            bytes_transferred -= dg.len();
-            send_q_.pop_front();
+            log_warn << "write_handler() called with empty send_q_. "
+                     << "Transport may not be reliable, closing the socket";
+            FAILED_HANDLER(asio::error_code(EPROTO,
+                                            asio::error::system_category));
         }
-        gcomm_assert(bytes_transferred == 0);
-
-        if (send_q_.empty() == false)
+        else if (send_q_.front().len() < bytes_transferred
+#ifdef GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+                 || ::rand() % bytes_transferred_less_than_rate == 0
+#endif // GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+            )
         {
-            const Datagram& dg(send_q_.front());
-            boost::array<asio::const_buffer, 2> cbs;
-            cbs[0] = asio::const_buffer(dg.header()
-                                        + dg.header_offset(),
-                                        dg.header_len());
-            cbs[1] = asio::const_buffer(&dg.payload()[0],
-                                        dg.payload().size());
-            write_one(cbs);
+            log_warn << "write_handler() bytes_transferred "
+                     << bytes_transferred
+                     << " less than sent "
+                     << send_q_.front().len()
+                     << ". Transport may not be reliable, closing the socket";
+            FAILED_HANDLER(asio::error_code(EPROTO,
+                                            asio::error::system_category));
         }
-        else if (state_ == S_CLOSING)
+        else
         {
-            log_debug << "deferred close of " << id();
-            close_socket();
-            state_ = S_CLOSED;
+            while (send_q_.empty() == false &&
+                   bytes_transferred >= send_q_.front().len())
+            {
+                const Datagram& dg(send_q_.front());
+                bytes_transferred -= dg.len();
+                send_q_.pop_front();
+            }
+            if (bytes_transferred != 0
+#ifdef GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+                 || ::rand() % bytes_transferred_not_zero_rate == 0
+#endif // GCOMM_ASIO_TCP_SIMULATE_WRITE_HANDLER_ERROR
+            )
+            {
+                log_warn << "write_handler() bytes_transferred "
+                         << bytes_transferred
+                         << " after processing the send_q_. "
+                         << "Transport may not be reliable, closing the socket";
+                FAILED_HANDLER(asio::error_code(EPROTO,
+                                                asio::error::system_category));
+            }
+            else if (send_q_.empty() == false)
+            {
+                const Datagram& dg(send_q_.front());
+                gu::array<asio::const_buffer, 2>::type cbs;
+                cbs[0] = asio::const_buffer(dg.header()
+                                            + dg.header_offset(),
+                                            dg.header_len());
+                cbs[1] = asio::const_buffer(&dg.payload()[0],
+                                            dg.payload().size());
+                write_one(cbs);
+            }
+            else if (state_ == S_CLOSING)
+            {
+                log_debug << "deferred close of " << id();
+                close_socket();
+                state_ = S_CLOSED;
+            }
         }
     }
     else if (state_ == S_CLOSING)
@@ -304,6 +343,37 @@ void gcomm::AsioTcpSocket::set_option(const std::string& key,
     }
 }
 
+namespace gcomm
+{
+    typedef gu::shared_ptr<gcomm::AsioTcpSocket>::type AsioTcpSocketPtr;
+    class AsioPostForSendHandler
+    {
+    public:
+        AsioPostForSendHandler(const AsioTcpSocketPtr& socket)
+            :
+            socket_(socket)
+        { }
+        void operator()()
+        {
+            Critical<AsioProtonet> crit(socket_->net_);
+            if (socket_->state() == gcomm::Socket::S_CONNECTED &&
+                socket_->send_q_.empty() == false)
+            {
+                const gcomm::Datagram& dg(socket_->send_q_.front());
+                gu::array<asio::const_buffer, 2>::type cbs;
+                cbs[0] = asio::const_buffer(dg.header()
+                                            + dg.header_offset(),
+                                            dg.header_len());
+                cbs[1] = asio::const_buffer(&dg.payload()[0],
+                                            dg.payload().size());
+                socket_->write_one(cbs);
+            }
+        }
+    private:
+        AsioTcpSocketPtr socket_;
+    };
+}
+
 int gcomm::AsioTcpSocket::send(const Datagram& dg)
 {
     Critical<AsioProtonet> crit(net_);
@@ -332,13 +402,7 @@ int gcomm::AsioTcpSocket::send(const Datagram& dg)
 
     if (send_q_.size() == 1)
     {
-        boost::array<asio::const_buffer, 2> cbs;
-        cbs[0] = asio::const_buffer(priv_dg.header()
-                                    + priv_dg.header_offset(),
-                                    priv_dg.header_len());
-        cbs[1] = asio::const_buffer(&priv_dg.payload()[0],
-                                    priv_dg.payload().size());
-        write_one(cbs);
+        net_.io_service_.post(AsioPostForSendHandler(shared_from_this()));
     }
     return 0;
 }
@@ -352,7 +416,8 @@ void gcomm::AsioTcpSocket::read_handler(const asio::error_code& ec,
     if (ec)
     {
 #ifdef HAVE_ASIO_SSL_HPP
-        if (ec.category() == asio::error::get_ssl_category())
+        if (ec.category() == asio::error::get_ssl_category() &&
+            gu::exclude_ssl_error(ec) == false)
         {
             log_warn << "read_handler(): " << ec.message() << " ("
                      << gu::extra_error_info(ec) << ")";
@@ -430,7 +495,7 @@ void gcomm::AsioTcpSocket::read_handler(const asio::error_code& ec,
         }
     }
 
-    boost::array<asio::mutable_buffer, 1> mbs;
+    gu::array<asio::mutable_buffer, 1>::type mbs;
     mbs[0] = asio::mutable_buffer(&recv_buf_[0] + recv_offset_,
                                   recv_buf_.size() - recv_offset_);
     read_one(mbs);
@@ -444,7 +509,8 @@ size_t gcomm::AsioTcpSocket::read_completion_condition(
     if (ec)
     {
 #ifdef HAVE_ASIO_SSL_HPP
-        if (ec.category() == asio::error::get_ssl_category())
+        if (ec.category() == asio::error::get_ssl_category() &&
+            gu::exclude_ssl_error(ec) == false)
         {
             log_warn << "read_completion_condition(): " << ec.message() << " ("
                      << gu::extra_error_info(ec) << ")";
@@ -491,7 +557,7 @@ void gcomm::AsioTcpSocket::async_receive()
 
     gcomm_assert(state() == S_CONNECTED);
 
-    boost::array<asio::mutable_buffer, 1> mbs;
+    gu::array<asio::mutable_buffer, 1>::type mbs;
 
     mbs[0] = asio::mutable_buffer(&recv_buf_[0], recv_buf_.size());
     read_one(mbs);
@@ -542,9 +608,9 @@ void gcomm::AsioTcpSocket::set_socket_options()
 #endif
 }
 
-void gcomm::AsioTcpSocket::read_one(boost::array<asio::mutable_buffer, 1>& mbs)
+void gcomm::AsioTcpSocket::read_one(
+    gu::array<asio::mutable_buffer, 1>::type& mbs)
 {
-#ifdef HAVE_ASIO_SSL_HPP
     if (ssl_socket_ != 0)
     {
         async_read(*ssl_socket_, mbs,
@@ -559,7 +625,6 @@ void gcomm::AsioTcpSocket::read_one(boost::array<asio::mutable_buffer, 1>& mbs)
     }
     else
     {
-#endif /* HAVE_ASIO_SSL_HPP */
         async_read(socket_, mbs,
                    boost::bind(&AsioTcpSocket::read_completion_condition,
                                shared_from_this(),
@@ -569,16 +634,13 @@ void gcomm::AsioTcpSocket::read_one(boost::array<asio::mutable_buffer, 1>& mbs)
                                shared_from_this(),
                                asio::placeholders::error,
                                asio::placeholders::bytes_transferred));
-#ifdef HAVE_ASIO_SSL_HPP
     }
-#endif /* HAVE_ASIO_SSL_HPP */
 }
 
 
 void gcomm::AsioTcpSocket::write_one(
-    const boost::array<asio::const_buffer, 2>& cbs)
+    const gu::array<asio::const_buffer, 2>::type& cbs)
 {
-#ifdef HAVE_ASIO_SSL_HPP
     if (ssl_socket_ != 0)
     {
         async_write(*ssl_socket_, cbs,
@@ -589,15 +651,12 @@ void gcomm::AsioTcpSocket::write_one(
     }
     else
     {
-#endif /* HAVE_ASIO_SSL_HPP */
         async_write(socket_, cbs,
                     boost::bind(&AsioTcpSocket::write_handler,
                                 shared_from_this(),
                                 asio::placeholders::error,
                                 asio::placeholders::bytes_transferred));
-#ifdef HAVE_ASIO_SSL_HPP
     }
-#endif /* HAVE_ASIO_SSL_HPP */
 }
 
 
@@ -605,7 +664,6 @@ void gcomm::AsioTcpSocket::close_socket()
 {
     try
     {
-#ifdef HAVE_ASIO_SSL_HPP
         if (ssl_socket_ != 0)
         {
             // close underlying transport before calling shutdown()
@@ -615,18 +673,14 @@ void gcomm::AsioTcpSocket::close_socket()
         }
         else
         {
-#endif /* HAVE_ASIO_SSL_HPP */
             socket_.close();
-#ifdef HAVE_ASIO_SSL_HPP
         }
-#endif /* HAVE_ASIO_SSL_HPP */
     }
     catch (...) { }
 }
 
 void gcomm::AsioTcpSocket::assign_local_addr()
 {
-#ifdef HAVE_ASIO_SSL_HPP
     if (ssl_socket_ != 0)
     {
         local_addr_ = gcomm::uri_string(
@@ -639,20 +693,16 @@ void gcomm::AsioTcpSocket::assign_local_addr()
     }
     else
     {
-#endif /* HAVE_ASIO_SSL_HPP */
         local_addr_ = gcomm::uri_string(
             gu::scheme::tcp,
             gu::escape_addr(socket_.local_endpoint().address()),
             gu::to_string(socket_.local_endpoint().port())
             );
-#ifdef HAVE_ASIO_SSL_HPP
     }
-#endif /* HAVE_ASIO_SSL_HPP */
 }
 
 void gcomm::AsioTcpSocket::assign_remote_addr()
 {
-#ifdef HAVE_ASIO_SSL_HPP
     if (ssl_socket_ != 0)
     {
         remote_addr_ = gcomm::uri_string(
@@ -665,15 +715,12 @@ void gcomm::AsioTcpSocket::assign_remote_addr()
     }
     else
     {
-#endif /* HAVE_ASIO_SSL_HPP */
         remote_addr_ = uri_string(
             gu::scheme::tcp,
             gu::escape_addr(socket_.remote_endpoint().address()),
             gu::to_string(socket_.remote_endpoint().port())
             );
-#ifdef HAVE_ASIO_SSL_HPP
     }
-#endif /* HAVE_ASIO_SSL_HPP */
 }
 
 
@@ -706,7 +753,6 @@ void gcomm::AsioTcpAcceptor::accept_handler(
             s->assign_local_addr();
             s->assign_remote_addr();
             s->set_socket_options();
-#ifdef HAVE_ASIO_SSL_HPP
             if (s->ssl_socket_ != 0)
             {
                 log_debug << "socket "
@@ -722,11 +768,8 @@ void gcomm::AsioTcpAcceptor::accept_handler(
             }
             else
             {
-#endif /* HAVE_ASIO_SSL_HP */
                 s->state_ = Socket::S_CONNECTED;
-#ifdef HAVE_ASIO_SSL_HPP
             }
-#endif /* HAVE_ASIO_SSL_HPP */
             accepted_socket_ = socket;
             log_debug << "accepted socket " << socket->id();
             net_.dispatch(id(), Datagram(), ProtoUpMeta(error.value()));
@@ -738,29 +781,17 @@ void gcomm::AsioTcpAcceptor::accept_handler(
             log_debug << "accept failed: " << e.what();
         }
         AsioTcpSocket* new_socket(new AsioTcpSocket(net_, uri_));
-#ifdef HAVE_ASIO_SSL_HPP
         if (uri_.get_scheme() == gu::scheme::ssl)
         {
             new_socket->ssl_socket_ =
                 new asio::ssl::stream<asio::ip::tcp::socket>(
                     net_.io_service_, net_.ssl_context_);
-            acceptor_.async_accept(new_socket->ssl_socket_->lowest_layer(),
-                                   boost::bind(&AsioTcpAcceptor::accept_handler,
-                                               this,
-                                               SocketPtr(new_socket),
-                                               asio::placeholders::error));
         }
-        else
-        {
-#endif /* HAVE_ASIO_SSL_HPP */
-            acceptor_.async_accept(new_socket->socket_,
-                                   boost::bind(&AsioTcpAcceptor::accept_handler,
-                                               this,
-                                               SocketPtr(new_socket),
-                                               asio::placeholders::error));
-#ifdef HAVE_ASIO_SSL_HPP
-        }
-#endif /* HAVE_ASIO_SSL_HPP */
+        acceptor_.async_accept(new_socket->socket(),
+                               boost::bind(&AsioTcpAcceptor::accept_handler,
+                                           this,
+                                           SocketPtr(new_socket),
+                                           asio::placeholders::error));
     }
     else
     {
@@ -786,29 +817,17 @@ void gcomm::AsioTcpAcceptor::listen(const gu::URI& uri)
         acceptor_.bind(*i);
         acceptor_.listen();
         AsioTcpSocket* new_socket(new AsioTcpSocket(net_, uri));
-#ifdef HAVE_ASIO_SSL_HPP
         if (uri_.get_scheme() == gu::scheme::ssl)
         {
             new_socket->ssl_socket_ =
                 new asio::ssl::stream<asio::ip::tcp::socket>(
                     net_.io_service_, net_.ssl_context_);
-            acceptor_.async_accept(new_socket->ssl_socket_->lowest_layer(),
-                                   boost::bind(&AsioTcpAcceptor::accept_handler,
-                                               this,
-                                               SocketPtr(new_socket),
-                                               asio::placeholders::error));
         }
-        else
-        {
-#endif /* HAVE_ASIO_SSL_HPP */
-            acceptor_.async_accept(new_socket->socket_,
-                                   boost::bind(&AsioTcpAcceptor::accept_handler,
-                                               this,
-                                               SocketPtr(new_socket),
-                                               asio::placeholders::error));
-#ifdef HAVE_ASIO_SSL_HPP
-        }
-#endif /* HAVE_ASIO_SSL_HPP */
+        acceptor_.async_accept(new_socket->socket(),
+                               boost::bind(&AsioTcpAcceptor::accept_handler,
+                                           this,
+                                           SocketPtr(new_socket),
+                                           asio::placeholders::error));
     }
     catch (asio::system_error& e)
     {
