@@ -25,12 +25,30 @@ WSREP_SST_OPT_DATA=""
 WSREP_SST_OPT_AUTH=${WSREP_SST_OPT_AUTH:-}
 WSREP_SST_OPT_USER=${WSREP_SST_OPT_USER:-}
 WSREP_SST_OPT_PSWD=${WSREP_SST_OPT_PSWD:-}
-WSREP_LOG_DEBUG=0
+WSREP_SST_OPT_VERSION=""
+
+WSREP_LOG_DEBUG=""
 
 while [ $# -gt 0 ]; do
 case "$1" in
     '--address')
         readonly WSREP_SST_OPT_ADDR="$2"
+        #
+        # Break address string into host:port/path parts
+        #
+        if echo $WSREP_SST_OPT_ADDR | grep -qe '^\[.*\]'
+        then
+            # IPv6 notation
+            readonly WSREP_SST_OPT_HOST=${WSREP_SST_OPT_ADDR/\]*/\]}
+            readonly WSREP_SST_OPT_HOST_UNESCAPED=$(echo $WSREP_SST_OPT_HOST | \
+                 cut -d '[' -f 2 | cut -d ']' -f 1)
+        else
+            # "traditional" notation
+            readonly WSREP_SST_OPT_HOST=${WSREP_SST_OPT_ADDR%%[:/]*}
+        fi
+        readonly WSREP_SST_OPT_PORT=$(echo $WSREP_SST_OPT_ADDR | \
+                cut -d ']' -f 2 | cut -s -d ':' -f 2 | cut -d '/' -f 1)
+        readonly WSREP_SST_OPT_PATH=${WSREP_SST_OPT_ADDR#*/}
         shift
         ;;
     '--bypass')
@@ -80,6 +98,10 @@ case "$1" in
         WSREP_SST_OPT_USER="$2"
         shift
         ;;
+    '--mysqld-version')
+        readonly WSREP_SST_OPT_VERSION="$2"
+        shift
+        ;;
     '--gtid')
         readonly WSREP_SST_OPT_GTID="$2"
         shift
@@ -99,7 +121,6 @@ readonly WSREP_SST_OPT_BYPASS
 readonly WSREP_SST_OPT_BINLOG
 readonly WSREP_SST_OPT_CONF_SUFFIX
 
-
 #
 # user can specify xtrabackup specific settings that will be used during sst
 # process like encryption, etc.....
@@ -114,6 +135,9 @@ parse_cnf()
     local var=$2
     local reval=""
 
+    # normalize the variable name by replacing all '_' with '-'
+    var=${var//_/-}
+
     # print the default settings for given group using my_print_default.
     # normalize the variable names specified in cnf file (user can use _ or - for example log-bin or log_bin)
     # then grep for needed variable
@@ -137,6 +161,39 @@ parse_cnf()
 }
 
 
+#
+# Converts an input string into a boolean "on", "off"
+# Converts:
+#   "on", "true", "1" -> "on" (case insensitive)
+#   "off", "false", "0" -> "off" (case insensitive)
+# All other values : -> default_value
+#
+# 1st param: input value
+# 2nd param: default value
+normalize_boolean()
+{
+    local input_value=$1
+    local default_value=$2
+    local return_value=$default_value
+
+    if [[ "$input_value" == "1" ]]; then
+        return_value="on"
+    elif [[ "$input_value" =~ ^[Oo][Nn]$ ]]; then
+        return_value="on"
+    elif [[ "$input_value" =~ ^[Tt][Rr][Uu][Ee]$ ]]; then
+        return_value="on"
+    elif [[ "$input_value" == "0" ]]; then
+        return_value="off"
+    elif [[ "$input_value" =~ ^[Oo][Ff][Ff]$ ]]; then
+        return_value="off"
+    elif [[ "$input_value" =~ ^[Ff][Aa][Ll][Ss][Ee]$ ]]; then
+        return_value="off"
+    fi
+
+    echo $return_value
+}
+
+
 # try to use my_print_defaults, mysql and mysqldump that come with the sources
 # (for MTR suite)
 SCRIPTS_DIR="$(cd $(dirname "$0"); pwd -P)"
@@ -169,12 +226,9 @@ wsrep_auth_not_set()
 }
 
 # For Bug:1200727
-if $MY_PRINT_DEFAULTS -c $WSREP_SST_OPT_CONF sst | grep -q "wsrep_sst_auth"
+if wsrep_auth_not_set
 then
-    if wsrep_auth_not_set
-    then
-        WSREP_SST_OPT_AUTH=$($MY_PRINT_DEFAULTS -c $WSREP_SST_OPT_CONF sst | grep -- "--wsrep_sst_auth" | cut -d= -f2)
-    fi
+    WSREP_SST_OPT_AUTH=$(parse_cnf sst wsrep-sst-auth "")
 fi
 readonly WSREP_SST_OPT_AUTH
 
@@ -195,11 +249,19 @@ else
     SST_PROGRESS_FILE=""
 fi
 
+
+WSREP_LOG_DEBUG=$(parse_cnf sst wsrep-debug "")
+if [ -z "$WSREP_LOG_DEBUG" ]; then
+    WSREP_LOG_DEBUG=$(parse_cnf mysqld wsrep-debug "")
+fi
+WSREP_LOG_DEBUG=$(normalize_boolean "$WSREP_LOG_DEBUG" "off")
+if [[ "$WSREP_LOG_DEBUG" == "off" ]]; then
+    WSREP_LOG_DEBUG=""
+fi
+
+
 # Determine the timezone to use for error logging
 LOGGING_TZ=$(parse_cnf mysqld log-timestamps "")
-if [ -z "$LOGGING_TZ" ]; then
-    LOGGING_TZ=$(parse_cnf mysqld log_timestamps "")
-fi
 if [ -z "$LOGGING_TZ" ]; then
     LOGGING_TZ="UTC"
 fi
@@ -268,4 +330,3 @@ wsrep_check_programs()
 
     return $ret
 }
-
