@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.
@@ -945,9 +945,19 @@ int replace_user_table(THD *thd, TABLE *table, LEX_USER *combo,
     optimize_plugin_compare_by_pointer(&combo->plugin);
     builtin_plugin= auth_plugin_is_built_in(combo->plugin.str);
 
+    /* The user record was neither present nor the intention was to create it */
     if (!can_create_user)
     {
-      my_error(ER_CANT_CREATE_USER_WITH_GRANT, MYF(0));
+      if(thd->lex->sql_command == SQLCOM_GRANT)
+      {
+        /* Have come here to GRANT privilege to the non-existing user */
+        my_error(ER_CANT_CREATE_USER_WITH_GRANT, MYF(0));
+      }
+      else if (update_password)
+      {
+        /* Have come here to update the password of the non-existing user */
+        my_error(ER_PASSWORD_NO_MATCH, MYF(0), combo->user.str, combo->host.str);
+      }
       error= 1;
       goto end;
     }
