@@ -350,7 +350,7 @@ bool wsrep_provider_update (sys_var *self, THD* thd, enum_var_type type)
      there can be several concurrent clients changing wsrep_provider
   */
   mysql_mutex_unlock(&LOCK_global_system_variables);
-  wsrep_stop_replication(thd);
+  wsrep_stop_replication(thd, false);
 
   /*
     Unlock and lock LOCK_wsrep_slave_threads to maintain lock order & avoid
@@ -492,7 +492,7 @@ bool wsrep_cluster_address_update (sys_var *self, THD* thd, enum_var_type type)
      there can be several concurrent clients changing wsrep_provider
   */
   mysql_mutex_unlock(&LOCK_global_system_variables);
-  wsrep_stop_replication(thd);
+  wsrep_stop_replication(thd, false);
   /*
     Unlock and lock LOCK_wsrep_slave_threads to maintain lock order & avoid
     any potential deadlock.
@@ -601,18 +601,15 @@ void wsrep_node_address_init (const char* value)
   wsrep_node_address = (value) ? my_strdup(key_memory_wsrep, value, MYF(0)) : NULL;
 }
 
-bool wsrep_slave_threads_check (sys_var *self, THD* thd, set_var* var)
+static void wsrep_slave_count_change_update ()
 {
-  mysql_mutex_lock(&LOCK_wsrep_slave_threads);
-  wsrep_slave_count_change += (var->save_result.ulonglong_value -
-                               wsrep_slave_threads);
-  mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
-
-  return false;
+  // wsrep_running_threads = appliers threads + 1 rollbacker thread
+  wsrep_slave_count_change = (wsrep_slave_threads - wsrep_running_threads + 1);
 }
 
 bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
 {
+  wsrep_slave_count_change_update();
   if (wsrep_slave_count_change > 0)
   {
     WSREP_DEBUG("Creating %d applier threads, total %ld", wsrep_slave_count_change, wsrep_slave_threads);
@@ -681,6 +678,19 @@ bool wsrep_desync_update (sys_var *self, THD* thd, enum_var_type type)
   return false;
 }
 
+bool wsrep_forced_binlog_format_check (sys_var *self, THD* thd, set_var* var)
+{
+  {
+    WSREP_WARN("Percona-XtraDB-Cluster has deprecated"
+               " wsrep_forced_binlog_format");
+    push_warning_printf(
+      thd, Sql_condition::SL_WARNING, ER_UNKNOWN_ERROR,
+      "Percona-XtraDB-Cluster has deprecated"
+      " wsrep_forced_binlog_format");
+  }
+  return false;
+}
+
 bool wsrep_max_ws_size_update (sys_var *self, THD *thd, enum_var_type)
 {
   char max_ws_size_opt[128];
