@@ -51,6 +51,11 @@ Created 1/8/1996 Heikki Tuuri
 #include "fsp0sysspace.h"
 #include "srv0start.h"
 
+//TODO:Robert - może to nie powinno być tutaj, includuje tylko dla fil_encryption_t
+#include "fil0fil.h"
+
+#include "fil0crypt.h" //dla FIL_ENCRYPTION_KEY_DEFAULT
+
 /*****************************************************************//**
 Based on a table object, this function builds the entry to be inserted
 in the SYS_TABLES system table.
@@ -365,7 +370,7 @@ dict_build_table_def_step(
 	trx_t*	trx = thr_get_trx(thr);
 	dict_table_assign_new_id(table, trx);
 
-	err = dict_build_tablespace_for_table(table);
+	err = dict_build_tablespace_for_table(table, node);
 	if (err != DB_SUCCESS) {
 		return(err);
 	}
@@ -382,7 +387,8 @@ dict_build_table_def_step(
 @return DB_SUCCESS or error code. */
 dberr_t
 dict_build_tablespace(
-	Tablespace*	tablespace)
+	Tablespace*	tablespace,
+	tab_node_t*	node)
 {
 	dberr_t		err	= DB_SUCCESS;
 	mtr_t		mtr;
@@ -415,7 +421,9 @@ dict_build_tablespace(
 		tablespace->name(),
 		datafile->filepath(),
 		tablespace->flags(),
-		FIL_IBD_FILE_INITIAL_SIZE);
+		FIL_IBD_FILE_INITIAL_SIZE,
+		node ? node->mode : FIL_ENCRYPTION_DEFAULT,
+		node ? node->create_info_encryption_key_id : CreateInfoEncryptionKeyId());
 	if (err != DB_SUCCESS) {
 		return(err);
 	}
@@ -435,7 +443,6 @@ dict_build_tablespace(
 	/* Once we allow temporary general tablespaces, we must do this;
 	mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO); */
 	ut_a(!FSP_FLAGS_GET_TEMPORARY(tablespace->flags()));
-
 	bool ret = fsp_header_init(space, FIL_IBD_FILE_INITIAL_SIZE, &mtr);
 	mtr_commit(&mtr);
 
@@ -451,7 +458,8 @@ dict_build_tablespace(
 @return DB_SUCCESS or error code */
 dberr_t
 dict_build_tablespace_for_table(
-	dict_table_t*	table)
+	dict_table_t*	table,
+        tab_node_t*	node)
 {
 	dberr_t		err	= DB_SUCCESS;
 	mtr_t		mtr;
@@ -470,6 +478,12 @@ dict_build_tablespace_for_table(
 			DICT_TF2_FLAG_UNSET(table,
 					    DICT_TF2_FTS_AUX_HEX_NAME););
 
+	if (node && (node->mode == FIL_ENCRYPTION_ON ||
+			(node->mode == FIL_ENCRYPTION_DEFAULT &&
+			 (srv_encrypt_tables == SRV_ENCRYPT_TABLES_ONLINE_TO_KEYRING
+			  || srv_encrypt_tables == SRV_ENCRYPT_TABLES_KEYRING_FORCE))))
+		DICT_TF2_FLAG_SET(table, DICT_TF2_ENCRYPTION);
+
 	if (needs_file_per_table) {
 		/* This table will need a new tablespace. */
 
@@ -531,7 +545,9 @@ dict_build_tablespace_for_table(
 
 		err = fil_ibd_create(
 			space, table->name.m_name, filepath, fsp_flags,
-			FIL_IBD_FILE_INITIAL_SIZE);
+			FIL_IBD_FILE_INITIAL_SIZE,
+			node ? node->mode : FIL_ENCRYPTION_DEFAULT,
+			node ? node->create_info_encryption_key_id : CreateInfoEncryptionKeyId());
 
 		ut_free(filepath);
 
@@ -999,7 +1015,7 @@ dict_create_index_tree_step(
 
 	mtr_start(&mtr);
 
-	const bool	missing = index->table->ibd_file_missing
+	const bool	missing = index->table->file_unreadable
 		|| dict_table_is_discarded(index->table);
 
 	if (!missing) {
@@ -1073,7 +1089,7 @@ dict_create_index_tree_in_mem(
 
 	/* Currently this function is being used by temp-tables only.
 	Import/Discard of temp-table is blocked and so this assert. */
-	ut_ad(index->table->ibd_file_missing == 0
+	ut_ad(index->table->file_unreadable == 0
 	      && !dict_table_is_discarded(index->table));
 
 	page_no = btr_create(
@@ -1356,7 +1372,9 @@ tab_create_graph_create(
 /*====================*/
 	dict_table_t*	table,	/*!< in: table to create, built as a memory data
 				structure */
-	mem_heap_t*	heap)	/*!< in: heap where created */
+	mem_heap_t*	heap,	/*!< in: heap where created */
+	fil_encryption_t mode,	/*!< in: encryption mode */
+	const CreateInfoEncryptionKeyId &create_info_encryption_key_id) /*!< in: encryption key_id */
 {
 	tab_node_t*	node;
 
@@ -1369,6 +1387,8 @@ tab_create_graph_create(
 
 	node->state = TABLE_BUILD_TABLE_DEF;
 	node->heap = mem_heap_create(256);
+        node->mode= mode;
+        node->create_info_encryption_key_id= create_info_encryption_key_id;
 
 	node->tab_def = ins_node_create(INS_DIRECT, dict_sys->sys_tables,
 					heap);
