@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2016, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 1995, 2018, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2009, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -55,12 +55,20 @@ struct log_group_t;
 /** Magic value to use instead of log checksums when they are disabled */
 #define LOG_NO_CHECKSUM_MAGIC 0xDEADBEEFUL
 
+/* Margin for the free space in the smallest log group, before a new query
+step which modifies the database, is started */
+
+#define LOG_CHECKPOINT_FREE_PER_THREAD	(4 * UNIV_PAGE_SIZE)
+#define LOG_CHECKPOINT_EXTRA_FREE	(8 * UNIV_PAGE_SIZE)
+
 typedef ulint (*log_checksum_func_t)(const byte* log_block);
 
 /** Pointer to the log checksum calculation function. Protected with
 log_sys->mutex. */
 extern log_checksum_func_t log_checksum_algorithm_ptr;
 
+static const char ib_logfile_basename[] = "ib_logfile";
+
 /*******************************************************************//**
 Calculates where in log files we find a specified lsn.
 @return log file number */
@@ -133,7 +141,28 @@ Closes the log.
 @return lsn */
 lsn_t
 log_close(void);
-/*===========*/
+
+/** Write the encryption info into the log file header(the 3rd block).
+It just need to flush the file header block with current master key.
+@param[in]	key	encryption key
+@param[in]	iv	encryption iv
+@param[in]	is_boot	if it is for bootstrap
+@return true if success. */
+bool
+log_write_encryption(
+	byte*	key,
+	byte*	iv);
+
+/** Rotate the redo log encryption
+ * It will re-encrypt the redo log encryption metadata and write it to
+ * redo log file header.
+ * @return true if success. */
+bool
+log_rotate_encryption();
+
+/** Enables redo log encryption. */
+void
+log_enable_encryption_if_set();
 /************************************************************//**
 Gets the current lsn.
 @return current lsn */
@@ -181,6 +210,12 @@ void
 log_io_complete(
 /*============*/
 	log_group_t*	group);	/*!< in: log group */
+/* Read the first log file header to get the encryption
+information if it exist.
+@return true if success */
+MY_NODISCARD
+bool
+log_read_encryption();
 /******************************************************//**
 This function is called, e.g., when a transaction wants to commit. It checks
 that the log has been written to the log file up to the last log entry written
@@ -360,9 +395,24 @@ ulint
 log_block_calc_checksum_crc32(
 	const byte*	block);
 
+/** Gets a log block encrypt bit.
+@param[in]	log_block	log block
+@return TRUE if this block was encrypted */
+MY_NODISCARD
+UNIV_INLINE
+bool
+log_block_get_encrypt_bit(
+	const byte*	log_block);
 /** Calculates the checksum for a log block using the "no-op" algorithm.
 @param[in]	block	the redo log block
 @return		the calculated checksum value */
+/** Sets the log block encrypt bit.
+@param[in,out]	log_block	log block
+@param[in]	val		value to set */
+UNIV_INLINE
+void
+log_block_set_encrypt_bit(byte* log_block, bool val);
+
 UNIV_INLINE
 ulint
 log_block_calc_checksum_none(const byte*	block);
@@ -503,6 +553,11 @@ extern my_bool	innodb_log_checksums;
 #define LOG_BLOCK_FLUSH_BIT_MASK 0x80000000UL
 					/* mask used to get the highest bit in
 					the preceding field */
+#define LOG_BLOCK_ENCRYPT_BIT_MASK 0x8000UL
+					/* mask used to get the highest bit in
+					the data len field, this bit is to
+					indicate if this block is encrypted or
+					not */
 #define	LOG_BLOCK_HDR_DATA_LEN	4	/* number of bytes of log written to
 					this block */
 #define	LOG_BLOCK_FIRST_REC_GROUP 6	/* offset of the first start of an
@@ -549,6 +604,7 @@ because InnoDB never supported more than one copy of the redo log. */
 LOG_FILE_START_LSN started here, 4 bytes earlier than LOG_HEADER_START_LSN,
 which the LOG_FILE_START_LSN was renamed to. */
 #define LOG_HEADER_PAD1		4
+#define LOG_HEADER_ENCRYPT_VER	4
 /** LSN of the start of data in this log file (with format version 1;
 in format version 0, it was called LOG_FILE_START_LSN and at offset 4). */
 #define LOG_HEADER_START_LSN	8
@@ -817,6 +873,14 @@ struct log_t{
 	mutex_exit(&log_sys->write_mutex);	\
 } while (0)
 
+/* log scrubbing speed, in bytes/sec */
+extern ulonglong innodb_scrub_log_speed;
+
+/** Event to wake up log_scrub_thread */
+extern os_event_t       log_scrub_event;
+/** Whether log_scrub_thread is active */
+extern bool             log_scrub_thread_active;
+
 /** Calculate the offset of an lsn within a log group.
 @param[in]	lsn	log sequence number
 @param[in]	group	log group
