@@ -39,11 +39,11 @@ Created 3/26/1996 Heikki Tuuri
 #include "ut0lst.h"
 #include "read0types.h"
 #include "page0types.h"
+#include "ut0mutex.h"
+#include "trx0trx.h"
 #ifdef WITH_WSREP
 #include "trx0xa.h"
 #endif /* WITH_WSREP */
-#include "ut0mutex.h"
-#include "trx0trx.h"
 
 typedef UT_LIST_BASE_NODE_T(trx_t) trx_ut_list_t;
 
@@ -281,7 +281,6 @@ trx_sys_update_mysql_binlog_offset(
         trx_sysf_t*     sys_header, /*!< in: trx sys header */
 #endif /* WITH_WSREP */
 	mtr_t*		mtr);	/*!< in: mtr */
-
 /*****************************************************************//**
 Prints to stderr the MySQL binlog offset info in the trx system header if
 the magic number shows it valid. */
@@ -294,7 +293,9 @@ void
 trx_sys_update_wsrep_checkpoint(
         const XID*      xid,         /*!< in: WSREP XID */
         trx_sysf_t*     sys_header,  /*!< in: sys_header */
-        mtr_t*          mtr);        /*!< in: mtr       */
+        mtr_t*          mtr,         /*!< in: mtr       */
+        bool            recovery = false);
+                                     /*!< in: running recovery */
 
 void
 /** Read WSREP checkpoint XID from sys header. */
