@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
@@ -46,7 +46,9 @@ Created 12/9/1995 Heikki Tuuri
 #include "buf0flu.h"
 #include "srv0srv.h"
 #include "log0recv.h"
+#include "lock0lock.h"
 #include "fil0fil.h"
+#include "fil0crypt.h"
 #include "dict0boot.h"
 #include "dict0stats_bg.h"
 #include "srv0srv.h"
@@ -58,6 +60,10 @@ Created 12/9/1995 Heikki Tuuri
 #include "sync0sync.h"
 #endif /* !UNIV_HOTBACKUP */
 
+#include "system_key.h"
+
+static redo_log_encrypt_enum found_log_encryption_mode = REDO_LOG_ENCRYPT_OFF;
+
 /*
 General philosophy of InnoDB redo-logs:
 
@@ -93,6 +99,10 @@ my_bool	innodb_log_checksums;
 /** Pointer to the log checksum calculation function */
 log_checksum_func_t log_checksum_algorithm_ptr;
 
+/* Next log block number to do dummy record filling if no log records written
+for a while */
+static ulint		next_lbn_to_pad = 0;
+
 /* These control how often we print warnings if the last checkpoint is too
 old */
 bool	log_has_printed_chkp_warning = false;
@@ -108,12 +118,6 @@ time_t	log_last_margine_warning_time;
 #define LOG_BUF_FLUSH_RATIO	2
 #define LOG_BUF_FLUSH_MARGIN	(LOG_BUF_WRITE_MARGIN + 4 * UNIV_PAGE_SIZE)
 
-/* Margin for the free space in the smallest log group, before a new query
-step which modifies the database, is started */
-
-#define LOG_CHECKPOINT_FREE_PER_THREAD	(4 * UNIV_PAGE_SIZE)
-#define LOG_CHECKPOINT_EXTRA_FREE	(8 * UNIV_PAGE_SIZE)
-
 /* This parameter controls asynchronous making of a new checkpoint; the value
 should be bigger than LOG_POOL_PREFLUSH_RATIO_SYNC */
 
@@ -130,6 +134,16 @@ the previous */
 #define LOG_UNLOCK_NONE_FLUSHED_LOCK	1
 #define LOG_UNLOCK_FLUSH_LOCK		2
 
+/** Event to wake up log_scrub_thread */
+os_event_t      log_scrub_event;
+/** Whether log_scrub_thread is active */
+bool            log_scrub_thread_active;
+
+extern "C"
+os_thread_ret_t
+DECLARE_THREAD(log_scrub_thread)(void*);
+
+
 /******************************************************//**
 Completes a checkpoint write i/o to a log file. */
 static
@@ -900,6 +914,7 @@ log_init(void)
 	/*----------------------------*/
 
 	log_sys->last_checkpoint_lsn = log_sys->lsn;
+	log_sys->next_checkpoint_lsn = log_sys->lsn;
 
 	rw_lock_create(
 		checkpoint_lock_key, &log_sys->checkpoint_lock,
@@ -923,6 +938,12 @@ log_init(void)
 
 	MONITOR_SET(MONITOR_LSN_CHECKPOINT_AGE,
 		    log_sys->lsn - log_sys->last_checkpoint_lsn);
+
+	log_scrub_thread_active= !srv_read_only_mode && srv_scrub_log;
+	if (log_scrub_thread_active) {
+		log_scrub_event= os_event_create("log_scrub_event");
+		os_thread_create(log_scrub_thread, NULL, NULL);
+	}
 }
 
 /******************************************************************//**
@@ -1077,6 +1098,448 @@ log_group_file_header_flush(
 	srv_stats.os_log_pending_writes.dec();
 }
 
+/* Read the first log file header to get the encryption. It's in the
+3rd block.
+@return true if success */
+bool
+log_read_encryption()
+{
+	byte		key[ENCRYPTION_KEY_LEN];
+	byte		iv[ENCRYPTION_KEY_LEN];
+
+	byte* log_block_buf_ptr = static_cast<byte*>(ut_malloc_nokey(
+		2 * OS_FILE_LOG_BLOCK_SIZE));
+	memset(log_block_buf_ptr, 0, 2 * OS_FILE_LOG_BLOCK_SIZE);
+	byte* log_block_buf = static_cast<byte*>(
+		ut_align(log_block_buf_ptr, OS_FILE_LOG_BLOCK_SIZE));
+
+	ulint		log_space_id = SRV_LOG_SPACE_FIRST_ID;
+	const page_id_t	page_id(log_space_id, 0);
+	fil_io(IORequestLogRead, true, page_id, univ_page_size,
+	       LOG_CHECKPOINT_1 + OS_FILE_LOG_BLOCK_SIZE,
+	       OS_FILE_LOG_BLOCK_SIZE, log_block_buf, NULL);
+
+	bool encryption_magic = false;
+	bool encrypted_log = false;
+
+	if (memcmp(log_block_buf + LOG_HEADER_CREATOR_END,
+		   ENCRYPTION_KEY_MAGIC_RK, ENCRYPTION_MAGIC_SIZE) == 0) {
+		encryption_magic = true;
+		found_log_encryption_mode = REDO_LOG_ENCRYPT_RK;
+
+		/* Make sure the keyring is loaded. */
+		if (!Encryption::check_keyring()) {
+			ut_free(log_block_buf_ptr);
+			ib::error()
+				<< "Redo log was encrypted,"
+				<< " but keyring plugin is not loaded.";
+			return(false);
+		}
+
+		unsigned char* info_ptr = log_block_buf + LOG_HEADER_CREATOR_END + ENCRYPTION_MAGIC_SIZE;
+	        uint version = mach_read_from_4(info_ptr);
+
+		memcpy(iv, info_ptr + ENCRYPTION_SERVER_UUID_LEN + 4, ENCRYPTION_KEY_LEN);
+
+#ifdef UNIV_ENCRYPT_DEBUG
+			fprintf(stderr, "Using redo log encryption key version: %u\n", version);
+#endif
+		
+		char *key_type = NULL;
+		char* rkey = NULL;
+		std::ostringstream percona_redo_with_ver_ss;
+		percona_redo_with_ver_ss << PERCONA_REDO_KEY_NAME << ':' << version;
+		size_t klen;
+		if (my_key_fetch(percona_redo_with_ver_ss.str().c_str(), &key_type, NULL,
+					reinterpret_cast<void**>(&rkey), &klen) ||
+				rkey == NULL)
+		{
+			ib::error() << "Couldn't fetch redo log encryption key: " << percona_redo_with_ver_ss.str() << ".";
+		} else if(key_type == NULL || strncmp(key_type, "AES", 3) != 0) {
+			ib::error() << "Unknown redo log encryption type: " << key_type << ".";
+		} else {
+			encrypted_log = true;
+			memcpy(key, rkey, ENCRYPTION_KEY_LEN);
+		}
+		my_free(key_type);
+		my_free(rkey);
+	}
+
+	if (memcmp(log_block_buf + LOG_HEADER_CREATOR_END,
+		   ENCRYPTION_KEY_MAGIC_V2, ENCRYPTION_MAGIC_SIZE) == 0) {
+		encryption_magic = true;
+		found_log_encryption_mode = REDO_LOG_ENCRYPT_MK;
+
+		/* Make sure the keyring is loaded. */
+		if (!Encryption::check_keyring()) {
+			ib::error()
+				<< "Redo log was encrypted,"
+				<< " but keyring plugin is not loaded.";
+		} else if (Encryption::decode_encryption_info(
+				key, iv,
+				log_block_buf + LOG_HEADER_CREATOR_END)) {
+			encrypted_log = true;
+		}
+
+	}
+	if (encrypted_log) {
+		/* If redo log encryption is enabled, set the
+		   space flag. Otherwise, we just fill the encryption
+		   information to space object for decrypting old
+		   redo log blocks. */
+		fil_space_t*	space = fil_space_get(log_space_id);
+		space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+		dberr_t err = fil_set_encryption(space->id,
+				Encryption::AES,
+				key, iv);
+
+		if (err == DB_SUCCESS) {
+			ut_free(log_block_buf_ptr);
+			ib::info() << "Read redo log encryption"
+				<< " metadata successful.";
+			return(true);
+		} else {
+			ut_free(log_block_buf_ptr);
+			ib::error()
+				<< "Can't set redo log tablespace"
+				<< " encryption metadata.";
+			return(false);
+		}
+	} else if (encryption_magic) {
+		ut_free(log_block_buf_ptr);
+		ib::error() << "Cannot read the encryption"
+			" information in log file header, please"
+			" check if keyring plugin loaded and"
+			" the key file exists.";
+		return(false);
+	}
+
+	ut_free(log_block_buf_ptr);
+	return(true);
+}
+
+
+/** Writes encryption information to log header.
+@param[in,out]	buf	log file header
+@param[in]	key	encryption key
+@param[in]	iv	encryption iv */
+static
+bool
+log_file_header_fill_encryption(
+	byte*		buf,
+	byte*		key,
+	byte*		iv)
+{
+	byte		encryption_info[ENCRYPTION_INFO_SIZE_V2];
+
+	if (!fsp_header_fill_encryption_info(key,
+				      iv, encryption_info)) {
+		return(false);
+	}
+
+	ut_ad(LOG_HEADER_CREATOR_END + ENCRYPTION_INFO_SIZE_V2
+	      < OS_FILE_LOG_BLOCK_SIZE);
+
+	memcpy(buf + LOG_HEADER_CREATOR_END,
+	       encryption_info,
+	       ENCRYPTION_INFO_SIZE_V2);
+
+	return(true);
+}
+
+static
+bool
+log_file_header_fill_encryption(
+	byte*		buf,
+	ulint           key_version,
+	byte*		iv)
+{
+	byte		encryption_info[ENCRYPTION_INFO_SIZE_V2] = {};
+
+	if (!fsp_header_fill_encryption_info(key_version, iv,
+				      encryption_info)) {
+		return(false);
+	}
+
+	ut_ad(LOG_HEADER_CREATOR_END + ENCRYPTION_INFO_SIZE_V2
+	      < OS_FILE_LOG_BLOCK_SIZE);
+
+	memcpy(buf + LOG_HEADER_CREATOR_END,
+	       encryption_info,
+	       ENCRYPTION_INFO_SIZE_V2);
+
+	return(true);
+}
+
+/** Write the encryption info into the log file header(the 3rd block).
+It just need to flush the file header block with current master key.
+@param[in]	key	encryption key
+@param[in]	iv	encryption iv
+@return true if success. */
+bool
+log_write_encryption(
+	byte*	key,
+	byte*	iv)
+{
+	const page_id_t	page_id(SRV_LOG_SPACE_FIRST_ID, 0);
+	byte*		log_block_buf_ptr;
+	byte*		log_block_buf;
+	ulint version = 1;
+
+	log_block_buf_ptr = static_cast<byte*>(ut_malloc_nokey(
+		2 * OS_FILE_LOG_BLOCK_SIZE));
+	memset(log_block_buf_ptr, 0, 2 * OS_FILE_LOG_BLOCK_SIZE);
+	log_block_buf = static_cast<byte*>(
+		ut_align(log_block_buf_ptr, OS_FILE_LOG_BLOCK_SIZE));
+
+	if (key == NULL && iv == NULL) {
+		fil_space_t*	space = fil_space_get(
+			SRV_LOG_SPACE_FIRST_ID);
+
+		key = space->encryption_key;
+		iv = space->encryption_iv;
+		version = space->encryption_key_version;
+	}
+
+	log_write_mutex_enter();
+	if (srv_redo_log_encrypt == REDO_LOG_ENCRYPT_MK || 
+	    found_log_encryption_mode == REDO_LOG_ENCRYPT_MK) {
+		if (!log_file_header_fill_encryption(log_block_buf,
+						     key,
+						     iv)) {
+			ut_free(log_block_buf_ptr);
+			log_write_mutex_exit();
+			return(false);
+		}
+	} else if (srv_redo_log_encrypt == REDO_LOG_ENCRYPT_RK ||
+		   found_log_encryption_mode == REDO_LOG_ENCRYPT_RK) {
+		if (!log_file_header_fill_encryption(log_block_buf,
+						     version,
+						     iv)) {
+			ut_free(log_block_buf_ptr);
+			log_write_mutex_exit();
+			return(false);
+		}
+	} else {
+		ut_ad(0);
+	}
+
+	log_sys->n_log_ios++;
+
+	MONITOR_INC(MONITOR_LOG_IO);
+
+	srv_stats.os_log_pending_writes.inc();
+
+	fil_io(IORequestLogWrite, true,
+	       page_id,
+	       univ_page_size,
+	       LOG_CHECKPOINT_1 + OS_FILE_LOG_BLOCK_SIZE,
+	       OS_FILE_LOG_BLOCK_SIZE, log_block_buf, NULL);
+
+	srv_stats.os_log_pending_writes.dec();
+	log_write_mutex_exit();
+
+	ut_free(log_block_buf_ptr);
+	return(true);
+}
+
+/** Rotate the redo log encryption
+It will re-encrypt the redo log encryption metadata and write it to
+redo log file header.
+@return true if success. */
+bool
+log_rotate_encryption() {
+	fil_space_t* space = fil_space_get(SRV_LOG_SPACE_FIRST_ID);
+ 	if (!FSP_FLAGS_GET_ENCRYPTION(space->flags)) {
+		return(true);
+	}
+ 	/* Rotate log tablespace */
+	return(log_write_encryption(NULL, NULL));
+}
+
+/** Check the redo log encryption is enabled or not.
+It will try to enable the redo log encryption and write the metadata to
+redo log file header. */
+void
+log_enable_encryption_if_set()
+{
+	fil_space_t* space = fil_space_get(SRV_LOG_SPACE_FIRST_ID);
+
+	if (srv_shutdown_state != SRV_SHUTDOWN_NONE) {
+		return;
+	}
+
+	/* Check encryption for redo log is enabled or not. If it's
+	enabled, we will start to encrypt the redo log block from now on.
+	Note: We need the server_uuid initialized, otherwise, the keyname will
+	not contains server uuid. */
+	if (srv_redo_log_encrypt != REDO_LOG_ENCRYPT_OFF
+	    && !FSP_FLAGS_GET_ENCRYPTION(space->flags)
+	    && strlen(server_uuid) > 0) {
+		byte	key[ENCRYPTION_KEY_LEN];
+		byte	iv[ENCRYPTION_KEY_LEN];
+
+		if (srv_read_only_mode) {
+			srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+			ib::error() << "Can't set redo log tablespace to be"
+				<< " encrypted in read-only mode.";
+			return;
+		}
+
+		bool encryption_enabled = false;
+		Encryption::random_value(iv);
+
+		uint version = 1;
+
+		if (srv_redo_log_encrypt == REDO_LOG_ENCRYPT_MK) {
+			Encryption::random_value(key);
+			encryption_enabled = true;
+			if (!log_write_encryption(key, iv)) {
+				srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+				ib::error() << "Can't set redo log"
+					<< " tablespace to be"
+					<< " encrypted.";
+				encryption_enabled = false;
+			}
+		} else if (srv_redo_log_encrypt == REDO_LOG_ENCRYPT_RK) {
+			// load latest key & write version
+
+			char *redo_key_type = NULL;
+			byte *rkey = NULL;
+			size_t   klen = 0;
+			
+			encryption_enabled = true;
+			
+			if (my_key_fetch(PERCONA_REDO_KEY_NAME, &redo_key_type, NULL,
+						reinterpret_cast<void**>(&rkey), &klen) || rkey == NULL)
+			{
+				if (my_key_generate(PERCONA_REDO_KEY_NAME, "AES", NULL, ENCRYPTION_KEY_LEN)) {
+					ib::error() << "Redo log key generation failed.";
+					encryption_enabled = false;
+				} else if (my_key_fetch(PERCONA_REDO_KEY_NAME, &redo_key_type, NULL,
+							reinterpret_cast<void**>(&rkey), &klen)) {
+					ib::error() << "Couldn't fetch newly generated redo key.";
+					encryption_enabled = false;
+				} else {
+					DBUG_ASSERT(rkey != NULL);
+					byte *rkey2 = NULL;
+					size_t   klen2 = 0;
+					bool err = (parse_system_key(rkey, klen, &version, &rkey2, &klen2) 
+						   == reinterpret_cast<uchar*>(NullS));
+					ut_ad(klen2 ==	ENCRYPTION_KEY_LEN);
+					if (err) {
+						encryption_enabled = false;
+					} else {
+						memcpy(key, rkey2, ENCRYPTION_KEY_LEN);
+					}
+					if (rkey2 != NULL) {
+						my_free(rkey2);
+					}
+				}
+			} else {
+				memcpy(key, rkey, ENCRYPTION_KEY_LEN);
+			}
+
+			if (encryption_enabled) {
+				ut_ad(redo_key_type && strcmp(redo_key_type, "AES") == 0);
+			}
+			if (redo_key_type != NULL) {
+				my_free(redo_key_type);
+			}
+			if (rkey != NULL) {
+				my_free(rkey);
+			}
+
+#ifdef UNIV_ENCRYPT_DEBUG
+			fprintf(stderr, "Fetched redo key: %s.\n", key);
+#endif
+
+			if (encryption_enabled && !log_write_encryption(key, iv)) {
+				srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+				ib::error() << "Can't set redo log"
+					<< " tablespace to be"
+					<< " encrypted.";
+				encryption_enabled = false;
+			}
+		}
+
+		if (encryption_enabled) {
+			space->flags |=
+				FSP_FLAGS_MASK_ENCRYPTION;
+			dberr_t err = fil_set_encryption(
+				space->id, Encryption::AES,
+				key, iv);
+			space->encryption_key_version = version;
+			if (err != DB_SUCCESS) {
+				srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+				ib::warn() << "Can't set redo log"
+					<< " tablespace to be"
+					<< " encrypted.";
+			} else {
+				ib::info() << "Redo log encryption is"
+					<< " enabled.";
+			}
+		}
+	}
+
+	/* If the redo log space is using default key, rotate it.
+	We also need the server_uuid initialized. */
+	if (space->encryption_type != Encryption::NONE
+	    && Encryption::master_key_id == ENCRYPTION_DEFAULT_MASTER_KEY_ID
+	    && !srv_read_only_mode
+	    && strlen(server_uuid) > 0
+	    && srv_redo_log_encrypt == REDO_LOG_ENCRYPT_MK) {
+			ut_ad(FSP_FLAGS_GET_ENCRYPTION(space->flags));
+
+			log_write_encryption(NULL, NULL);
+	}
+
+	if (space->encryption_type != Encryption::NONE
+	    && space->encryption_key_version ==REDO_LOG_ENCRYPT_NO_VERSION 
+	    && !srv_read_only_mode
+	    && strlen(server_uuid) > 0
+	    && srv_redo_log_encrypt == REDO_LOG_ENCRYPT_RK) {
+			/* This only happens when the server uuid was just generated, so we can
+			 * save the key to the keyring */
+			if (my_key_store(PERCONA_REDO_KEY_NAME, "AES", NULL,
+						space->encryption_key, ENCRYPTION_KEY_LEN))
+			{
+				srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+				ib::error() << "Can't store redo log encryption key.";
+			}
+			uint version = 0;
+			size_t   klen = 0;
+			size_t   klen2 = 0;
+			char *redo_key_type = NULL;
+			byte *rkey = NULL;
+			unsigned char *rkey2 = NULL;
+			if (my_key_fetch(PERCONA_REDO_KEY_NAME, &redo_key_type, NULL,
+						reinterpret_cast<void**>(&rkey), &klen)) {
+				srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+				ib::error() << "Can't fetch latest redo log encryption key.";
+			}
+			const bool err = (parse_system_key(rkey, klen, &version, &rkey2, &klen2) == reinterpret_cast<uchar*>(NullS));
+			if (err) {
+				srv_redo_log_encrypt = REDO_LOG_ENCRYPT_OFF;
+				ib::error() << "Can't parse latest redo log encryption key.";
+			}
+			space->encryption_key_version = version;
+			if (!log_write_encryption(NULL, NULL)) {
+				ib::error() << "Can't write redo log encryption information.";
+			}
+			if (rkey != NULL) {
+				my_free(rkey);
+			}
+			if (rkey2 != NULL) {
+				my_free(rkey2);
+			}
+			if (redo_key_type != NULL) {
+				my_free(redo_key_type);
+			}
+
+		}
+}
+
 /******************************************************//**
 Stores a 4-byte checksum to the trailer checksum field of a log block
 before writing it to a log file. This checksum is used in recovery to
@@ -1600,6 +2063,7 @@ log_complete_checkpoint(void)
 
 	log_sys->next_checkpoint_no++;
 
+	ut_ad(log_sys->next_checkpoint_lsn >= log_sys->last_checkpoint_lsn);
 	log_sys->last_checkpoint_lsn = log_sys->next_checkpoint_lsn;
 	MONITOR_SET(MONITOR_LSN_CHECKPOINT_AGE,
 		    log_sys->lsn - log_sys->last_checkpoint_lsn);
@@ -1652,6 +2116,7 @@ log_group_checkpoint(
 
 	ut_ad(!srv_read_only_mode);
 	ut_ad(log_mutex_own());
+	ut_ad(srv_shutdown_state != SRV_SHUTDOWN_LAST_PHASE);
 #if LOG_CHECKPOINT_SIZE > OS_FILE_LOG_BLOCK_SIZE
 # error "LOG_CHECKPOINT_SIZE > OS_FILE_LOG_BLOCK_SIZE"
 #endif
@@ -1665,6 +2130,11 @@ log_group_checkpoint(
 	buf = group->checkpoint_buf;
 	memset(buf, 0, OS_FILE_LOG_BLOCK_SIZE);
 
+#ifdef UNIV_DEBUG
+	lsn_t		old_next_checkpoint_lsn
+		= mach_read_from_8(buf + LOG_CHECKPOINT_LSN);
+	ut_ad(old_next_checkpoint_lsn <= log_sys->next_checkpoint_lsn);
+#endif /* UNIV_DEBUG */
 	mach_write_to_8(buf + LOG_CHECKPOINT_NO, log_sys->next_checkpoint_no);
 	mach_write_to_8(buf + LOG_CHECKPOINT_LSN, log_sys->next_checkpoint_lsn);
 
@@ -1940,6 +2410,7 @@ log_checkpoint(
 		return(false);
 	}
 
+	ut_ad(oldest_lsn >= log_sys->next_checkpoint_lsn);
 	log_sys->next_checkpoint_lsn = oldest_lsn;
 	log_write_checkpoint_info(sync);
 	ut_ad(!log_mutex_own());
@@ -2223,7 +2694,7 @@ loop:
 
 	active_thd = srv_get_active_thread_type();
 
-	if (active_thd != SRV_NONE) {
+	if (active_thd != SRV_NONE || srv_n_fil_crypt_threads_started) {
 
 		if (active_thd == SRV_PURGE) {
 			srv_purge_wakeup();
@@ -2239,11 +2710,9 @@ loop:
 
 			switch (active_thd) {
 			case SRV_NONE:
-				/* This shouldn't happen because we've
-				already checked for this case before
-				entering the if(). We handle it here
-				to avoid a compiler warning. */
-				ut_error;
+				ut_ad(srv_n_fil_crypt_threads_started);
+				thread_type = "encryption thread";
+				break;
 			case SRV_WORKER:
 				thread_type = "worker threads";
 				break;
@@ -2267,16 +2736,25 @@ loop:
 	/* At this point only page_cleaner should be active. We wait
 	here to let it complete the flushing of the buffer pools
 	before proceeding further. */
+	os_rmb;
+	ut_ad(buf_lru_manager_running_threads == srv_buf_pool_instances
+	      || buf_lru_manager_running_threads == 0);
 	srv_shutdown_state = SRV_SHUTDOWN_FLUSH_PHASE;
 	count = 0;
-	while (buf_page_cleaner_is_active) {
-		++count;
-		os_thread_sleep(100000);
-		if (srv_print_verbose_log && count > 600) {
+	while (buf_page_cleaner_is_active
+	       || buf_lru_manager_running_threads > 0) {
+
+		if (srv_print_verbose_log && count == 0) {
 			ib::info() << "Waiting for page_cleaner to"
 				" finish flushing of buffer pool";
+		}
+		++count;
+		os_thread_sleep(100000);
+		if (count > 600) {
 			count = 0;
 		}
+
+		os_rmb;
 	}
 
 	log_mutex_enter();
@@ -2284,7 +2762,7 @@ loop:
 	const ulint	n_flush	= log_sys->n_pending_flushes;
 	log_mutex_exit();
 
-	if (n_write != 0 || n_flush != 0) {
+	if (log_scrub_thread_active || n_write != 0 || n_flush != 0) {
 		if (srv_print_verbose_log && count > 600) {
 			ib::info() << "Pending checkpoint_writes: " << n_write
 				<< ". Pending log flush writes: " << n_flush;
@@ -2295,6 +2773,13 @@ loop:
 
 	pending_io = buf_pool_check_no_pending_io();
 
+        if (log_scrub_thread_active) {
+                ut_ad(!srv_read_only_mode);
+                os_event_set(log_scrub_event);
+        }
+
+        ut_ad(!log_scrub_thread_active);
+
 	if (pending_io) {
 		if (srv_print_verbose_log && count > 600) {
 			ib::info() << "Waiting for " << pending_io << " buffer"
@@ -2363,7 +2848,14 @@ loop:
 
 	lsn = log_sys->lsn;
 
-	const bool	is_last = (lsn == log_sys->last_checkpoint_lsn)
+	/** If innodb_force_recovery is set to 6 then log_sys doesn't
+	have recent checkpoint information. So last checkpoint lsn
+	will never be equal to current lsn. */
+	const bool      is_last =
+		((srv_force_recovery == SRV_FORCE_NO_LOG_REDO
+		  && lsn == log_sys->last_checkpoint_lsn
+		  + LOG_BLOCK_HDR_SIZE)
+		 || lsn == log_sys->last_checkpoint_lsn)
 		&& (!srv_track_changed_pages
 		    || tracked_lsn == log_sys->last_checkpoint_lsn);
 
@@ -2421,6 +2913,8 @@ loop:
 	ut_a(freed);
 
 	ut_a(lsn == log_sys->lsn);
+	ut_ad(srv_force_recovery >= SRV_FORCE_NO_LOG_REDO
+	      || lsn == log_sys->last_checkpoint_lsn);
 
 	if (lsn < srv_start_lsn) {
 		ib::error() << "Log sequence number at shutdown " << lsn
@@ -2609,6 +3103,10 @@ log_shutdown(void)
 	mutex_free(&log_sys->write_mutex);
 	mutex_free(&log_sys->log_flush_order_mutex);
 
+	if (!srv_read_only_mode && srv_scrub_log) {
+		os_event_destroy(log_scrub_event);
+	}
+
 	recv_sys_close();
 }
 
@@ -2625,4 +3123,91 @@ log_mem_free(void)
 		log_sys = NULL;
 	}
 }
+
+static
+void
+log_pad_current_log_block(void)
+/*===========================*/
+{
+
+        ut_ad(!recv_no_log_write);
+        lsn_t lsn = log_reserve_and_open(OS_FILE_LOG_BLOCK_SIZE);
+
+        ulint pad_length = OS_FILE_LOG_BLOCK_SIZE
+                - log_sys->buf_free % OS_FILE_LOG_BLOCK_SIZE
+		- LOG_BLOCK_TRL_SIZE;
+        if (pad_length == (OS_FILE_LOG_BLOCK_SIZE - LOG_BLOCK_HDR_SIZE
+		- LOG_BLOCK_TRL_SIZE)) {
+                pad_length = 0;
+        }
+
+        if (pad_length) {
+                srv_stats.n_log_scrubs.inc();
+        }
+
+        for (ulint i = 0; i < pad_length; i++) {
+		byte b = MLOG_DUMMY_RECORD;
+                log_write_low(&b, 1);
+        }
+
+        lsn = log_sys->lsn;
+
+        log_close();
+
+        ut_a(lsn % OS_FILE_LOG_BLOCK_SIZE == LOG_BLOCK_HDR_SIZE);
+}
+
+/*****************************************************************//*
+If no log record has been written for a while, fill current log
+block with dummy records. */
+static
+void
+log_scrub()
+/*=========*/
+{
+        log_mutex_enter();
+        ulint cur_lbn = log_block_convert_lsn_to_no(log_sys->lsn);
+
+        if (next_lbn_to_pad == cur_lbn)
+        {
+                log_pad_current_log_block();
+        }
+
+        next_lbn_to_pad = log_block_convert_lsn_to_no(log_sys->lsn);
+        log_mutex_exit();
+}
+
+/* log scrubbing speed, in bytes/sec */
+ulonglong innodb_scrub_log_speed;
+
+/*****************************************************************//**
+This is the main thread for log scrub. It waits for an event and
+when waked up fills current log block with dummy records and
+sleeps again.
+@return this function does not return, it calls os_thread_exit() */
+extern "C"
+os_thread_ret_t
+DECLARE_THREAD(log_scrub_thread)(void*)
+{
+        ut_ad(!srv_read_only_mode);
+
+        while (srv_shutdown_state < SRV_SHUTDOWN_FLUSH_PHASE) {
+                /* log scrubbing interval in µs. */
+                ulonglong interval = 1000*1000*512/innodb_scrub_log_speed;
+
+                os_event_wait_time(log_scrub_event, static_cast<ulint>(interval));
+
+                log_scrub();
+
+                os_event_reset(log_scrub_event);
+        }
+
+        log_scrub_thread_active = false;
+
+        /* We count the number of threads in os_thread_exit(). A created
+        thread should always use that to exit and not use return() to exit. */
+        os_thread_exit();
+
+        OS_THREAD_DUMMY_RETURN;
+}
 #endif /* !UNIV_HOTBACKUP */
