@@ -43,16 +43,14 @@ Created 2011/04/18 Sunny Bains
 #include "srv0srv.h"
 #include "btr0types.h"
 #include "trx0trx.h"
+
 #ifdef WITH_WSREP
+#include "row0mysql.h"
+#include "dict0dict.h"
+
 extern "C" int wsrep_trx_is_aborting(void *thd_ptr);
 extern my_bool wsrep_debug;
 #endif /* WITH_WSREP */
-#include "row0mysql.h"
-#include "dict0dict.h"
-#ifdef WITH_WSREP
-extern "C" int wsrep_trx_is_aborting(void *thd_ptr);
-extern my_bool wsrep_debug;
-#endif
 
 /** Number of times a thread is allowed to enter InnoDB within the same
 SQL query after it has once got the ticket. */
@@ -230,7 +228,7 @@ srv_conc_enter_innodb_with_atomics(
 		}
 
 		os_thread_sleep(sleep_in_us);
-		trx->innodb_que_wait_timer += sleep_in_us;
+		trx->stats.bump_innodb_enter_wait(*trx, sleep_in_us);
 
 		trx->op_info = "";
 
@@ -322,13 +320,6 @@ srv_conc_force_exit_innodb(
 
 		return;
 	}
-#ifdef WITH_WSREP
-	if (wsrep_on(trx->mysql_thd) && 
-	    wsrep_trx_is_aborting(trx->mysql_thd)) {
-		srv_conc_force_enter_innodb(trx);
-		return;
-	}
-#endif /* WITH_WSREP */
 
 	srv_conc_exit_innodb_with_atomics(trx);
 
