@@ -919,7 +919,7 @@ trx_purge_mark_undo_for_truncate(
 	b. At-least 2 UNDO redo rseg/undo logs (besides the default rseg-0)
 	b. At-least 1 UNDO tablespace size > threshold. */
 	if (srv_undo_tablespaces_active < 2
-	    || (srv_undo_logs < (1 + srv_tmp_undo_logs + 2))) {
+	    || (srv_rollback_segments < (1 + srv_tmp_undo_logs + 2))) {
 		return;
 	}
 
@@ -1309,6 +1309,7 @@ trx_purge_rseg_get_next_history_log(
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 
+#ifdef UNIV_DEBUG
 		trx_sys_mutex_enter();
 
 		/* Add debug code to track history list corruption reported
@@ -1322,14 +1323,19 @@ trx_purge_rseg_get_next_history_log(
 		if (trx_sys->rseg_history_len > 2000000) {
 			ib::warn() << "Purge reached the head of the history"
 				" list, but its length is still reported as "
-				<< trx_sys->rseg_history_len << "! Make"
-				" a detailed bug report, and submit it to"
-				" http://bugs.mysql.com";
-			ut_ad(0);
+				<< trx_sys->rseg_history_len << " which is"
+				" unusually high.";
+			ib::info() << "This can happen for multiple reasons";
+			ib::info() << "1. A long running transaction is"
+				" withholding purging of undo logs or a read"
+				" view is open. Please try to commit the long"
+				" running transaction.";
+			ib::info() << "2. Try increasing the number of purge"
+				" threads to expedite purging of undo logs.";
 		}
 
 		trx_sys_mutex_exit();
-
+#endif
 		return;
 	}
 
