@@ -219,6 +219,23 @@ ut_time_ms(void)
 }
 #endif /* !UNIV_HOTBACKUP */
 
+/** Returns the number of milliseconds since some epoch using monotonic clock.
+The value may wrap around.  This should not be used as an accurate Time Of Day.
+It is only intended to be used as a means of calculating transient elapsed or
+projected time that will not be influenced by changes to the systems real time
+clock.  Returns a small structure that contains the result so as to poison the
+code and reveal any changes that might later be introduced by upstream.
+*/
+ut_monotonic_time
+ut_monotonic_time_ms(void) {
+	timespec	  tp;
+	ut_monotonic_time ret;
+	clock_gettime(CLOCK_MONOTONIC, &tp);
+
+	ret.ms = (ulint) tp.tv_sec * 1000 + tp.tv_nsec / 1000000;
+	return ret;
+}
+
 /**********************************************************//**
 Returns the difference of two times in seconds.
 @return time2 - time1 expressed in seconds */
@@ -855,7 +872,11 @@ ut_strerr(
 		       "of stored column");
 	case DB_COMPUTE_VALUE_FAILED:
 		return("Compute generated column failed");
-
+        case DB_DECRYPTION_FAILED:
+		return("Table is encrypted but decrypt failed.");
+	case DB_PAGE_CORRUPTED:
+		return("Page read from tablespace is corrupted.");
+        
 	/* do not add default: in order to produce a warning if new code
 	is added to the enum but not added here */
 	}
