@@ -24,7 +24,7 @@
 #include <unordered_map>
 
 /* MySQL header files */
-#include "./my_sys.h"
+#include "my_sys.h"
 #include "mysql/plugin.h"
 
 /* RocksDB header files */
@@ -55,7 +55,7 @@ public:
   //         or other Status on failure.
   // If returned status is OK, TransactionDB will eventually call UnLock().
   virtual rocksdb::Status TryLockFor(int64_t timeout_time
-                                     __attribute__((__unused__))) override;
+                                     MY_ATTRIBUTE((__unused__))) override;
 
   // Unlock Mutex that was successfully locked by Lock() or TryLockUntil()
   virtual void UnLock() override;
