@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -235,6 +235,12 @@ vio_set_cert_stuff(SSL_CTX *ctx, const char *cert_file, const char *key_file,
 }
 
 #ifndef HAVE_YASSL
+
+/*
+  OpenSSL 1.1 supports native platform threads,
+  so we don't need the following callback functions.
+*/
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 /* OpenSSL specific */
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
 
@@ -398,16 +404,16 @@ static void deinit_lock_callback_functions()
 }
 #endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
+
 void vio_ssl_end()
 {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-  int i= 0;
-#endif
-
   if (ssl_initialized) {
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-    ERR_remove_state(0);
-#endif
+    int i;
+
+    ERR_remove_thread_state(0);
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
     ERR_free_strings();
     EVP_cleanup();
 
@@ -416,11 +422,11 @@ void vio_ssl_end()
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
     deinit_lock_callback_functions();
 
-    for (; i < CRYPTO_num_locks(); ++i)
+    for (i= 0; i < CRYPTO_num_locks(); ++i)
       mysql_rwlock_destroy(&openssl_stdlocks[i].lock);
     OPENSSL_free(openssl_stdlocks);
-#endif
 
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
     ssl_initialized= FALSE;
   }
 }
@@ -437,9 +443,11 @@ void ssl_start()
     OpenSSL_add_all_algorithms();
     SSL_load_error_strings();
 
-#if !defined(HAVE_YASSL) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
+#ifndef HAVE_YASSL
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     init_ssl_locks();
     init_lock_callback_functions();
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 #endif
   }
 }
